@@ -243,7 +243,15 @@ int pj_get_suggested_operation(PJ_CONTEXT*,
             // onshore. So in a general way, prefer a onshore area to a
             // offshore one.
             if( iBest < 0 ||
-                (alt.accuracy >= 0 && alt.accuracy < bestAccuracy &&
+                (alt.accuracy >= 0 &&
+                (alt.accuracy < bestAccuracy ||
+                 // If two operations have the same accuracy, use the one that
+                 // is contained within a larger one
+                 (alt.accuracy == bestAccuracy &&
+                  alt.minxSrc > opList[iBest].minxSrc &&
+                  alt.minySrc > opList[iBest].minySrc &&
+                  alt.maxxSrc < opList[iBest].maxxSrc &&
+                  alt.maxySrc < opList[iBest].maxySrc)) &&
                 !alt.isOffshore) ) {
                 iBest = i;
                 bestAccuracy = alt.accuracy;
@@ -346,6 +354,8 @@ similarly, but prefers the 2D resp. 3D interfaces if available.
                         if (proj_log_level(P->ctx, PJ_LOG_TELL) >= PJ_LOG_DEBUG) {
                             std::string msg("Using coordinate operation ");
                             msg += alt.name;
+                            msg += " as a fallback due to lack of more "
+                                   "appropriate operations";
                             pj_log(P->ctx, PJ_LOG_DEBUG, msg.c_str());
                         }
                         P->iCurCoordOp = i;
@@ -774,11 +784,8 @@ PJ *pj_create_internal (PJ_CONTEXT *ctx, const char *definition) {
     It may even use free formatting "proj  =  utm;  zone  =32  ellps= GRS80".
     Note that the semicolon separator is allowed, but not required.
 **************************************************************************************/
-    PJ    *P;
     char  *args, **argv;
     size_t argc, n;
-    int    ret;
-    int    allow_init_epsg;
 
     if (nullptr==ctx)
         ctx = pj_get_default_ctx ();
@@ -806,19 +813,11 @@ PJ *pj_create_internal (PJ_CONTEXT *ctx, const char *definition) {
         return nullptr;
     }
 
-    /* ...and let pj_init_ctx do the hard work */
-    /* New interface: forbid init=epsg:XXXX syntax by default */
-    allow_init_epsg = proj_context_get_use_proj4_init_rules(ctx, FALSE);
-    P = pj_init_ctx_with_allow_init_epsg (ctx, (int) argc, argv, allow_init_epsg);
+    PJ* P = pj_create_argv_internal (ctx, (int) argc, argv);
 
     free (argv);
     free (args);
 
-    /* Support cs2cs-style modifiers */
-    ret = cs2cs_emulation_setup  (P);
-    if (0==ret)
-        return proj_destroy (P);
-
     return P;
 }
 
@@ -857,11 +856,8 @@ indicator, as in {"+proj=utm", "+zone=32"}, or leave it out, as in {"proj=utm",
 /*************************************************************************************/
 PJ *pj_create_argv_internal (PJ_CONTEXT *ctx, int argc, char **argv) {
 /**************************************************************************************
-Same as proj_create_argv() but calls pj_create_internal() instead of proj_create() internally
+For use by pipeline init function.
 **************************************************************************************/
-    PJ *P;
-    const char *c;
-
     if (nullptr==ctx)
         ctx = pj_get_default_ctx ();
     if (nullptr==argv) {
@@ -869,16 +865,16 @@ Same as proj_create_argv() but calls pj_create_internal() instead of proj_create
         return nullptr;
     }
 
-    /* We assume that free format is used, and build a full proj_create compatible string */
-    c = pj_make_args (argc, argv);
-    if (nullptr==c) {
-        proj_context_errno_set(ctx, PROJ_ERR_OTHER /*ENOMEM*/);
-        return nullptr;
-    }
+    /* ...and let pj_init_ctx do the hard work */
+    /* New interface: forbid init=epsg:XXXX syntax by default */
+    const int allow_init_epsg = proj_context_get_use_proj4_init_rules(ctx, FALSE);
+    PJ* P = pj_init_ctx_with_allow_init_epsg (ctx, argc, argv, allow_init_epsg);
 
-    P = pj_create_internal (ctx, c);
+    /* Support cs2cs-style modifiers */
+    int ret = cs2cs_emulation_setup  (P);
+    if (0==ret)
+        return proj_destroy (P);
 
-    free ((char *) c);
     return P;
 }
 
@@ -970,6 +966,534 @@ std::string pj_add_type_crs_if_needed(const std::string& str)
     return ret;
 }
 
+
+// ---------------------------------------------------------------------------
+static double simple_min(const double* data, const int arr_len) {
+    double min_value = data[0];
+    for( int iii = 1; iii < arr_len; iii++ ) {
+        if (data[iii] < min_value)
+            min_value = data[iii];
+    }
+    return min_value;
+}
+
+
+// ---------------------------------------------------------------------------
+static double simple_max(const double* data, const int arr_len) {
+    double max_value = data[0];
+    for( int iii = 1; iii < arr_len; iii++ ) {
+        if ((data[iii] > max_value || max_value == HUGE_VAL) && data[iii] != HUGE_VAL)
+            max_value = data[iii];
+    }
+    return max_value;
+ }
+
+
+// ---------------------------------------------------------------------------
+static int _find_previous_index(const int iii, const double* data, const int arr_len) {
+    // find index of nearest valid previous value if exists
+    int prev_iii = iii - 1;
+    if (prev_iii == -1)  // handle wraparound
+        prev_iii = arr_len - 1;
+    while (data[prev_iii] == HUGE_VAL && prev_iii != iii) {
+        prev_iii --;
+        if (prev_iii == -1)  // handle wraparound
+            prev_iii = arr_len - 1;
+    }
+    return prev_iii;
+}
+
+
+// ---------------------------------------------------------------------------
+/******************************************************************************
+Handles the case when longitude values cross the antimeridian
+when calculating the minimum.
+Note: The data array must be in a linear ring.
+Note: This requires a densified ring with at least 2 additional
+        points per edge to correctly handle global extents.
+If only 1 additional point:
+    |        |
+    |RL--x0--|RL--
+    |        |
+-180    180|-180
+If they are evenly spaced and it crosses the antimeridian:
+x0 - L = 180
+R - x0 = -180
+For example:
+Let R = -179.9, x0 = 0.1, L = -179.89
+x0 - L = 0.1 - -179.9 = 180
+R - x0 = -179.89 - 0.1 ~= -180
+This is the same in the case when it didn't cross the antimeridian.
+If you have 2 additional points:
+    |            |
+    |RL--x0--x1--|RL--
+    |            |
+-180        180|-180
+If they are evenly spaced and it crosses the antimeridian:
+x0 - L = 120
+x1 - x0 = 120
+R - x1 = -240
+For example:
+Let R = -179.9, x0 = -59.9, x1 = 60.1 L = -179.89
+x0 - L = 59.9 - -179.9 = 120
+x1 - x0 = 60.1 - 59.9 = 120
+R - x1 = -179.89 - 60.1 ~= -240
+However, if they are evenly spaced and it didn't cross the antimeridian:
+x0 - L = 120
+x1 - x0 = 120
+R - x1 = 120
+From this, we have a delta that is guaranteed to be significantly
+large enough to tell the difference reguarless of the direction
+the antimeridian was crossed.
+However, even though the spacing was even in the source projection, it isn't
+guaranteed in the target geographic projection. So, instead of 240, 200 is used
+as it significantly larger than 120 to be sure that the antimeridian was crossed
+but smalller than 240 to account for possible irregularities in distances
+when re-projecting. Also, 200 ensures latitudes are ignored for axis order handling.
+******************************************************************************/
+static double antimeridian_min(const double* data, const int arr_len) {
+    double positive_min = HUGE_VAL;
+    double min_value = HUGE_VAL;
+    int crossed_meridian_count = 0;
+    bool positive_meridian = false;
+
+    for( int iii = 0; iii < arr_len; iii++ ) {
+        if (data[iii] == HUGE_VAL)
+            continue;
+        int prev_iii = _find_previous_index(iii, data, arr_len);
+        // check if crossed meridian
+        double delta = data[prev_iii] - data[iii];
+        // 180 -> -180
+        if (delta >= 200 && delta != HUGE_VAL) {
+            if (crossed_meridian_count == 0)
+                positive_min = min_value;
+            crossed_meridian_count ++;
+            positive_meridian = false;
+        // -180 -> 180
+        } else if (delta <= -200 && delta != HUGE_VAL) {
+            if (crossed_meridian_count == 0)
+                positive_min = data[iii];
+            crossed_meridian_count ++;
+            positive_meridian = true;
+        }
+        // positive meridian side min
+        if (positive_meridian && data[iii] < positive_min)
+            positive_min = data[iii];
+        // track general min value
+        if (data[iii] < min_value)
+            min_value = data[iii];
+    }
+
+    if (crossed_meridian_count == 2)
+        return positive_min;
+    else if (crossed_meridian_count == 4)
+        // bounds extends beyond -180/180
+        return -180;
+    return min_value;
+}
+
+
+// ---------------------------------------------------------------------------
+// Handles the case when longitude values cross the antimeridian
+// when calculating the minimum.
+// Note: The data array must be in a linear ring.
+// Note: This requires a densified ring with at least 2 additional
+//       points per edge to correctly handle global extents.
+// See antimeridian_min docstring for reasoning.
+static double antimeridian_max(const double* data, const int arr_len) {
+    double negative_max = -HUGE_VAL;
+    double max_value = -HUGE_VAL;
+    bool negative_meridian = false;
+    int crossed_meridian_count = 0;
+
+    for( int iii = 0; iii < arr_len; iii++ ) {
+        if (data[iii] == HUGE_VAL)
+            continue;
+        int prev_iii = _find_previous_index(iii, data, arr_len);
+        // check if crossed meridian
+        double delta = data[prev_iii] - data[iii];
+        // 180 -> -180
+        if (delta >= 200 && delta != HUGE_VAL) {
+            if (crossed_meridian_count == 0)
+                negative_max = data[iii];
+            crossed_meridian_count ++;
+            negative_meridian = true;
+        // -180 -> 180
+        } else if (delta <= -200 && delta != HUGE_VAL){
+            if (crossed_meridian_count == 0)
+                negative_max = max_value;
+            negative_meridian = false;
+            crossed_meridian_count++;
+        }
+        // negative meridian side max
+        if (negative_meridian
+            && (data[iii] > negative_max || negative_max == HUGE_VAL)
+            && data[iii] != HUGE_VAL
+        )
+            negative_max = data[iii];
+        // track general max value
+        if ((data[iii] > max_value || max_value == HUGE_VAL) && data[iii] != HUGE_VAL)
+            max_value = data[iii];
+    }
+    if (crossed_meridian_count == 2)
+        return negative_max;
+    else if (crossed_meridian_count == 4)
+        // bounds extends beyond -180/180
+        return 180;
+    return max_value;
+}
+
+
+// ---------------------------------------------------------------------------
+// Check if the original projected bounds contains
+// the north pole.
+// This assumes that the destination CRS is geographic.
+static bool contains_north_pole(
+    PJ* projobj,
+    PJ_DIRECTION pj_direction,
+    const double xmin,
+    const double ymin,
+    const double xmax,
+    const double ymax,
+    bool lon_lat_order
+) {
+    double pole_y = 90;
+    double pole_x = 0;
+    if (!lon_lat_order) {
+        pole_y = 0;
+        pole_x = 90;
+    }
+    proj_trans_generic(
+        projobj,
+        opposite_direction(pj_direction),
+        &pole_x, sizeof(double), 1,
+        &pole_y, sizeof(double), 1,
+        nullptr, sizeof(double), 0,
+        nullptr, sizeof(double), 0
+    );
+    if (xmin < pole_x && pole_x < xmax && ymax > pole_y && pole_y > ymin)
+        return true;
+    return false;
+}
+
+
+// ---------------------------------------------------------------------------
+// Check if the original projected bounds contains
+// the south pole.
+// This assumes that the destination CRS is geographic.
+static bool contains_south_pole(
+    PJ* projobj,
+    PJ_DIRECTION pj_direction,
+    const double xmin,
+    const double ymin,
+    const double xmax,
+    const double ymax,
+    bool lon_lat_order
+) {
+    double pole_y = -90;
+    double pole_x = 0;
+    if (!lon_lat_order) {
+        pole_y = 0;
+        pole_x = -90;
+    }
+    proj_trans_generic(
+        projobj,
+        opposite_direction(pj_direction),
+        &pole_x, sizeof(double), 1,
+        &pole_y, sizeof(double), 1,
+        nullptr, sizeof(double), 0,
+        nullptr, sizeof(double), 0
+    );
+    if (xmin < pole_x && pole_x < xmax && ymax > pole_y && pole_y > ymin)
+        return true;
+    return false;
+}
+
+
+// ---------------------------------------------------------------------------
+// Check if the target CRS of the transformation
+// has the longitude latitude axis order.
+// This assumes that the destination CRS is geographic.
+static int target_crs_lon_lat_order(
+    PJ_CONTEXT* transformer_ctx,
+    PJ* transformer_pj,
+    PJ_DIRECTION pj_direction
+) {
+    PJ* target_crs = nullptr;
+    if (pj_direction == PJ_FWD)
+        target_crs = proj_get_target_crs(transformer_ctx, transformer_pj);
+    else if (pj_direction == PJ_INV)
+        target_crs = proj_get_source_crs(transformer_ctx, transformer_pj);
+    if (target_crs == nullptr) {
+        proj_context_log_debug(transformer_ctx, "Unable to retrieve target CRS");
+        return -1;
+    }
+    PJ* coord_system_pj = proj_crs_get_coordinate_system(
+        transformer_ctx,
+        target_crs
+    );
+    proj_destroy(target_crs);
+    if (coord_system_pj == nullptr) {
+        proj_context_log_debug(transformer_ctx, "Unable to get target CRS coordinate system.");
+        return -1;
+    }
+    const char* abbrev = nullptr;
+    int success = proj_cs_get_axis_info(
+        transformer_ctx,
+        coord_system_pj,
+        0,
+        nullptr,
+        &abbrev,
+        nullptr,
+        nullptr,
+        nullptr,
+        nullptr,
+        nullptr
+    );
+    proj_destroy(coord_system_pj);
+    if (success != 1)
+        return -1;
+    return strcmp(abbrev, "lon") == 0 || strcmp(abbrev, "Lon") == 0;
+}
+
+// ---------------------------------------------------------------------------
+
+/** \brief Transform boundary,
+ *
+ * Transform boundary densifying the edges to account for nonlinear
+ * transformations along these edges and extracting the outermost bounds.
+ *
+ * If the destination CRS is geographic, the first axis is longitude,
+ * and xmax < xmin then the bounds crossed the antimeridian.
+ * In this scenario there are two polygons, one on each side of the antimeridian.
+ * The first polygon should be constructed with (xmin, ymin, 180, ymax)
+ * and the second with (-180, ymin, xmax, ymax).
+ *
+ * If the destination CRS is geographic, the first axis is latitude,
+ * and ymax < ymin then the bounds crossed the antimeridian.
+ * In this scenario there are two polygons, one on each side of the antimeridian.
+ * The first polygon should be constructed with (ymin, xmin, ymax, 180)
+ * and the second with (ymin, -180, ymax, xmax).
+ *
+ * @param context The PJ_CONTEXT object.
+ * @param P The PJ object representing the transformation.
+ * @param direction The direction of the transformation.
+ * @param xmin Minimum bounding coordinate of the first axis in source CRS
+ *             (target CRS if direction is inverse).
+ * @param ymin Minimum bounding coordinate of the second axis in source CRS.
+ *             (target CRS if direction is inverse).
+ * @param xmax Maximum bounding coordinate of the first axis in source CRS.
+ *             (target CRS if direction is inverse).
+ * @param ymax Maximum bounding coordinate of the second axis in source CRS.
+ *             (target CRS if direction is inverse).
+ * @param out_xmin Minimum bounding coordinate of the first axis in target CRS
+ *             (source CRS if direction is inverse).
+ * @param out_ymin Minimum bounding coordinate of the second axis in target CRS.
+ *             (source CRS if direction is inverse).
+ * @param out_xmax Maximum bounding coordinate of the first axis in target CRS.
+ *             (source CRS if direction is inverse).
+ * @param out_ymax Maximum bounding coordinate of the second axis in target CRS.
+ *             (source CRS if direction is inverse).
+ * @param densify_pts Recommended to use 21. This is the number of points
+ *     to use to densify the bounding polygon in the transformation.
+ * @return an integer. 1 if successful. 0 if failures encountered.
+ * @since 8.2
+ */
+int proj_trans_bounds(PJ_CONTEXT* context,
+                      PJ *P,
+                      PJ_DIRECTION direction,
+                      const double xmin,
+                      const double ymin,
+                      const double xmax,
+                      const double ymax,
+                      double* out_xmin,
+                      double* out_ymin,
+                      double* out_xmax,
+                      double* out_ymax,
+                      const int densify_pts
+) {
+    *out_xmin = HUGE_VAL;
+    *out_ymin = HUGE_VAL;
+    *out_xmax = HUGE_VAL;
+    *out_ymax = HUGE_VAL;
+
+    if (P == nullptr) {
+        proj_log_error(P, _("NULL P object not allowed."));
+        proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        return false;
+    }
+    if (densify_pts < 0 || densify_pts > 10000) {
+        proj_log_error(P, _("densify_pts must be between 0-10000."));
+        proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        return false;
+    }
+
+    PJ_PROJ_INFO pj_info = proj_pj_info(P);
+    if (pj_info.id == nullptr) {
+        proj_log_error(P, _("NULL transformation not allowed,"));
+        proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        return false;
+    }
+    if (strcmp(pj_info.id, "noop") == 0 || direction == PJ_IDENT) {
+        *out_xmin = xmin;
+        *out_xmax = xmax;
+        *out_ymin = ymin;
+        *out_ymax = ymax;
+        return true;
+    }
+
+    bool degree_output = proj_degree_output(P, direction) != 0;
+    bool degree_input = proj_degree_input(P, direction) != 0;
+    if (degree_output && densify_pts < 2) {
+        proj_log_error(P, _("densify_pts must be at least 2 if the output is geograpic."));
+        proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        return false;
+    }
+
+    int side_pts = densify_pts + 1;  // add one because we are densifying
+    const int boundary_len = side_pts * 4;
+    std::vector<double> x_boundary_array;
+    std::vector<double> y_boundary_array;
+    try
+    {
+        x_boundary_array.resize(boundary_len);
+        y_boundary_array.resize(boundary_len);
+    }
+    catch( const std::exception & e ) // memory allocation failure
+    {
+        proj_log_error(P, e.what());
+        proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        return false;
+    }
+    double delta_x = 0;
+    double delta_y = 0;
+    bool north_pole_in_bounds = false;
+    bool south_pole_in_bounds = false;
+    bool input_lon_lat_order = false;
+    bool output_lon_lat_order = false;
+    if (degree_input) {
+        int in_order_lon_lat = target_crs_lon_lat_order(
+            context, P, opposite_direction(direction)
+        );
+        if (in_order_lon_lat == -1)
+            return false;
+        input_lon_lat_order = in_order_lon_lat != 0;
+    }
+    if (degree_output) {
+        int out_order_lon_lat = target_crs_lon_lat_order(context, P, direction);
+        if (out_order_lon_lat == -1)
+            return false;
+        output_lon_lat_order = out_order_lon_lat != 0;
+        north_pole_in_bounds = contains_north_pole(
+            P,
+            direction,
+            xmin,
+            ymin,
+            xmax,
+            ymax,
+            output_lon_lat_order
+        );
+        south_pole_in_bounds = contains_south_pole(
+            P,
+            direction,
+            xmin,
+            ymin,
+            xmax,
+            ymax,
+            output_lon_lat_order
+        );
+    }
+
+    if (degree_input && xmax < xmin) {
+        if (!input_lon_lat_order) {
+            proj_log_error(P, _("latitude max < latitude min."));
+            proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+            return false;
+        }
+        // handle antimeridian
+        delta_x = (xmax - xmin + 360.0) / side_pts;
+    } else {
+        delta_x = (xmax - xmin) / side_pts;
+    }
+    if (degree_input && ymax < ymin) {
+        if (input_lon_lat_order) {
+            proj_log_error(P, _("latitude max < latitude min."));
+            proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+            return false;
+        }
+        // handle antimeridian
+        delta_y = (ymax - ymin + 360.0) / side_pts;
+    } else {
+        delta_y = (ymax - ymin) / side_pts;
+    }
+
+
+    // build densified bounding box
+    // Note: must be a linear ring for antimeridian logic
+    for( int iii = 0; iii < side_pts; iii++ )
+    {
+        // xmin boundary
+        y_boundary_array[iii] = ymax - iii * delta_y;
+        x_boundary_array[iii] = xmin;
+        // ymin boundary
+        y_boundary_array[iii + side_pts] = ymin;
+        x_boundary_array[iii + side_pts] = xmin + iii * delta_x;
+        // xmax boundary
+        y_boundary_array[iii + side_pts * 2] = ymin + iii * delta_y;
+        x_boundary_array[iii + side_pts * 2] = xmax;
+        // ymax boundary
+        y_boundary_array[iii + side_pts * 3] = ymax;
+        x_boundary_array[iii + side_pts * 3] = xmax - iii * delta_x;
+    }
+    proj_trans_generic (
+        P,
+        direction,
+        &x_boundary_array[0], sizeof(double), boundary_len,
+        &y_boundary_array[0], sizeof(double), boundary_len,
+        nullptr, 0, 0,
+        nullptr, 0, 0
+    );
+
+    if (!degree_output) {
+        *out_xmin = simple_min(&x_boundary_array[0], boundary_len);
+        *out_xmax = simple_max(&x_boundary_array[0], boundary_len);
+        *out_ymin = simple_min(&y_boundary_array[0], boundary_len);
+        *out_ymax = simple_max(&y_boundary_array[0], boundary_len);
+    } else if (north_pole_in_bounds && output_lon_lat_order) {
+        *out_xmin = -180;
+        *out_ymin = simple_min(&y_boundary_array[0], boundary_len);
+        *out_xmax = 180;
+        *out_ymax = 90;
+    } else if (north_pole_in_bounds) {
+        *out_xmin = simple_min(&x_boundary_array[0], boundary_len);
+        *out_ymin = -180;
+        *out_xmax = 90;
+        *out_ymax = 180;
+    } else if (south_pole_in_bounds && output_lon_lat_order) {
+        *out_xmin = -180;
+        *out_ymin = -90;
+        *out_xmax = 180;
+        *out_ymax = simple_max(&y_boundary_array[0], boundary_len);
+    } else if (south_pole_in_bounds) {
+        *out_xmin = -90;
+        *out_ymin = -180;
+        *out_xmax = simple_max(&x_boundary_array[0], boundary_len);
+        *out_ymax = 180;
+    } else if (output_lon_lat_order) {
+        *out_xmin = antimeridian_min(&x_boundary_array[0], boundary_len);
+        *out_xmax = antimeridian_max(&x_boundary_array[0], boundary_len);
+        *out_ymin = simple_min(&y_boundary_array[0], boundary_len);
+        *out_ymax = simple_max(&y_boundary_array[0], boundary_len);
+    } else {
+        *out_xmin = simple_min(&x_boundary_array[0], boundary_len);
+        *out_xmax = simple_max(&x_boundary_array[0], boundary_len);
+        *out_ymin = antimeridian_min(&y_boundary_array[0], boundary_len);
+        *out_ymax = antimeridian_max(&y_boundary_array[0], boundary_len);
+    }
+    return true;
+}
+
+
 /*****************************************************************************/
 static void reproject_bbox(PJ* pjGeogToCrs,
                           double west_lon, double south_lat,
@@ -993,25 +1517,31 @@ static void reproject_bbox(PJ* pjGeogToCrs,
         maxx = -maxx;
         maxy = -maxy;
 
-        std::vector<double> x(21 * 4), y(21 * 4);
-        for( int j = 0; j <= 20; j++ )
+        constexpr int N_STEPS = 20;
+        constexpr int N_STEPS_P1 = N_STEPS+1;
+        constexpr int XY_SIZE = N_STEPS_P1 * 4;
+        std::vector<double> x(XY_SIZE);
+        std::vector<double> y(XY_SIZE);
+        const double step_lon = (east_lon - west_lon) / N_STEPS;
+        const double step_lat = (north_lat - south_lat) / N_STEPS;
+        for( int j = 0; j <= N_STEPS; j++ )
         {
-            x[j] = west_lon + j * (east_lon - west_lon) / 20;
+            x[j] = west_lon + j * step_lon;
             y[j] = south_lat;
-            x[21+j] = west_lon + j * (east_lon - west_lon) / 20;
-            y[21+j] = north_lat;
-            x[21*2+j] = west_lon;
-            y[21*2+j] = south_lat + j * (north_lat - south_lat) / 20;
-            x[21*3+j] = east_lon;
-            y[21*3+j] = south_lat + j * (north_lat - south_lat) / 20;
+            x[N_STEPS_P1+j] = x[j];
+            y[N_STEPS_P1+j] = north_lat;
+            x[N_STEPS_P1*2+j] = west_lon;
+            y[N_STEPS_P1*2+j] = south_lat + j * step_lat;
+            x[N_STEPS_P1*3+j] = east_lon;
+            y[N_STEPS_P1*3+j] = y[N_STEPS_P1*2+j];
         }
         proj_trans_generic (
             pjGeogToCrs, PJ_FWD,
-                &x[0], sizeof(double), 21 * 4,
-                &y[0], sizeof(double), 21 * 4,
+                &x[0], sizeof(double), XY_SIZE,
+                &y[0], sizeof(double), XY_SIZE,
                 nullptr, 0, 0,
                 nullptr, 0, 0);
-        for( int j = 0; j < 21 * 4; j++ )
+        for( int j = 0; j < XY_SIZE; j++ )
         {
             if( x[j] != HUGE_VAL && y[j] != HUGE_VAL )
             {
@@ -1082,15 +1612,24 @@ static PJ* create_operation_to_geog_crs(PJ_CONTEXT* ctx, const PJ* crs) {
         geodetic_crs_type == PJ_TYPE_GEOGRAPHIC_2D_CRS ||
         geodetic_crs_type == PJ_TYPE_GEOGRAPHIC_3D_CRS )
     {
-        auto datum = proj_crs_get_datum(ctx, geodetic_crs);
-        auto datum_ensemble = proj_crs_get_datum_ensemble(ctx, geodetic_crs);
+        auto datum = proj_crs_get_datum_forced(ctx, geodetic_crs);
+        assert( datum );
         auto cs = proj_create_ellipsoidal_2D_cs(
             ctx, PJ_ELLPS2D_LONGITUDE_LATITUDE, nullptr, 0);
-        auto temp = proj_create_geographic_crs_from_datum(
-            ctx, "unnamed crs", datum ? datum : datum_ensemble,
-            cs);
+        auto ellps = proj_get_ellipsoid(ctx, datum);
         proj_destroy(datum);
-        proj_destroy(datum_ensemble);
+        double semi_major_metre = 0;
+        double inv_flattening = 0;
+        proj_ellipsoid_get_parameters(ctx, ellps, &semi_major_metre,
+                                      nullptr, nullptr, &inv_flattening);
+        // It is critical to set the prime meridian to 0
+        auto temp = proj_create_geographic_crs(
+            ctx, "unnamed crs", "unnamed datum",
+            proj_get_name(ellps),
+            semi_major_metre, inv_flattening,
+            "Reference prime meridian", 0, nullptr, 0,
+            cs);
+        proj_destroy(ellps);
         proj_destroy(cs);
         proj_destroy(geodetic_crs);
         geodetic_crs = temp;
@@ -1304,6 +1843,7 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
     const char* authority = nullptr;
     double accuracy = -1;
     bool allowBallparkTransformations = true;
+    bool forceOver = false;
     for (auto iter = options; iter && iter[0]; ++iter) {
         const char *value;
         if ((value = getOptionValue(*iter, "AUTHORITY="))) {
@@ -1320,7 +1860,13 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
                             "Invalid value for ALLOW_BALLPARK option.");
                 return nullptr;
             }
-        } else {
+        }
+        else if ((value = getOptionValue(*iter, "FORCE_OVER="))) {
+            if (ci_equal(value, "yes")) {
+                forceOver = true;
+            }
+        }
+        else {
             std::string msg("Unknown option :");
             msg += *iter;
             ctx->logger(ctx->logger_app_data, PJ_LOG_ERROR, msg.c_str());
@@ -1374,6 +1920,8 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
         return nullptr;
     }
 
+    ctx->forceOver = forceOver;
+
     PJ* P = proj_list_get(ctx, op_list, 0);
     assert(P);
 
@@ -1381,11 +1929,14 @@ PJ  *proj_create_crs_to_crs_from_pj (PJ_CONTEXT *ctx, const PJ *source_crs, cons
         proj_get_type(source_crs) == PJ_TYPE_GEOCENTRIC_CRS ||
         proj_get_type(target_crs) == PJ_TYPE_GEOCENTRIC_CRS ) {
         proj_list_destroy(op_list);
+        ctx->forceOver = false;
         return P;
     }
 
     auto preparedOpList = pj_create_prepared_operations(ctx, source_crs, target_crs,
                                                    op_list);
+
+    ctx->forceOver = false;
     proj_list_destroy(op_list);
 
     if( preparedOpList.empty() )
@@ -1609,7 +2160,7 @@ PJ_INFO proj_info (void) {
 
     /* build search path string */
     auto ctx = pj_get_default_ctx();
-    if (!ctx || ctx->search_paths.empty()) {
+    if (ctx->search_paths.empty()) {
         const auto searchpaths = pj_get_default_searchpaths(ctx);
         for( const auto& path: searchpaths ) {
             buf = path_append(buf, path.c_str(), &buf_size);
@@ -1623,8 +2174,8 @@ PJ_INFO proj_info (void) {
     free(const_cast<char*>(info.searchpath));
     info.searchpath = buf ? buf : empty;
 
-    info.paths = ctx ? ctx->c_compat_paths : nullptr;
-    info.path_count = ctx ? static_cast<int>(ctx->search_paths.size()) : 0;
+    info.paths = ctx->c_compat_paths;
+    info.path_count = static_cast<int>(ctx->search_paths.size());
 
     pj_release_lock ();
     return info;
@@ -1725,24 +2276,25 @@ PJ_GRID_INFO proj_grid_info(const char *gridname) {
         strncpy (grinfo.gridname, gridname, sizeof(grinfo.gridname) - 1);
 
         /* full path of grid */
-        pj_find_file(ctx, gridname, grinfo.filename, sizeof(grinfo.filename) - 1);
+        if( pj_find_file(ctx, gridname, grinfo.filename, sizeof(grinfo.filename) - 1) )
+        {
+            /* grid format */
+            strncpy (grinfo.format, format.c_str(), sizeof(grinfo.format) - 1);
 
-        /* grid format */
-        strncpy (grinfo.format, format.c_str(), sizeof(grinfo.format) - 1);
+            /* grid size */
+            grinfo.n_lon = grid.width();
+            grinfo.n_lat = grid.height();
 
-        /* grid size */
-        grinfo.n_lon = grid.width();
-        grinfo.n_lat = grid.height();
+            /* cell size */
+            grinfo.cs_lon = extent.resX;
+            grinfo.cs_lat = extent.resY;
 
-        /* cell size */
-        grinfo.cs_lon = extent.resX;
-        grinfo.cs_lat = extent.resY;
-
-        /* bounds of grid */
-        grinfo.lowerleft.lam  = extent.west;
-        grinfo.lowerleft.phi  = extent.south;
-        grinfo.upperright.lam = extent.east;
-        grinfo.upperright.phi = extent.north;
+            /* bounds of grid */
+            grinfo.lowerleft.lam  = extent.west;
+            grinfo.lowerleft.phi  = extent.south;
+            grinfo.upperright.lam = extent.east;
+            grinfo.upperright.phi = extent.north;
+        }
     };
 
     {
@@ -1894,6 +2446,48 @@ PJ_FACTORS proj_factors(PJ *P, PJ_COORD lp) {
     if (nullptr==P)
         return factors;
 
+    const auto type = proj_get_type(P);
+
+    if( type == PJ_TYPE_PROJECTED_CRS )
+    {
+        // If it is a projected CRS, then compute the factors on the conversion
+        // associated to it. We need to start from a temporary geographic CRS
+        // using the same datum as the one of the projected CRS, and with
+        // input coordinates being in longitude, latitude order in radian,
+        // to be consistent with the expectations of the lp input parameter.
+
+        auto ctx = P->ctx;
+        auto geodetic_crs = proj_get_source_crs(ctx, P);
+        assert(geodetic_crs);
+        auto datum = proj_crs_get_datum(ctx, geodetic_crs);
+        auto datum_ensemble = proj_crs_get_datum_ensemble(ctx, geodetic_crs);
+        auto cs = proj_create_ellipsoidal_2D_cs(
+            ctx, PJ_ELLPS2D_LONGITUDE_LATITUDE, "Radian", 1.0);
+        auto temp = proj_create_geographic_crs_from_datum(
+            ctx, "unnamed crs", datum ? datum : datum_ensemble,
+            cs);
+        proj_destroy(datum);
+        proj_destroy(datum_ensemble);
+        proj_destroy(cs);
+        proj_destroy(geodetic_crs);
+        auto newOp = proj_create_crs_to_crs_from_pj(ctx, temp, P, nullptr, nullptr);
+        proj_destroy(temp);
+        assert(newOp);
+        auto ret = proj_factors(newOp, lp);
+        proj_destroy(newOp);
+        return ret;
+    }
+
+    if( type != PJ_TYPE_CONVERSION &&
+        type != PJ_TYPE_TRANSFORMATION &&
+        type != PJ_TYPE_CONCATENATED_OPERATION &&
+        type != PJ_TYPE_OTHER_COORDINATE_OPERATION )
+    {
+        proj_log_error(P, _("Invalid type for P object"));
+        proj_errno_set (P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+        return factors;
+    }
+
     if (pj_factors(lp.lp, P, 0.0, &f))
         return factors;
 
