@@ -81,6 +81,17 @@ BaseObjectNNPtr::~BaseObjectNNPtr() = default;
 
 // ---------------------------------------------------------------------------
 
+//! @cond Doxygen_Suppress
+// cppcheck-suppress operatorEqVarError
+BaseObject &BaseObject::operator=(BaseObject &&) {
+    d->self_.reset();
+    return *this;
+}
+
+//! @endcond
+
+// ---------------------------------------------------------------------------
+
 /** Keep a reference to ourselves as an internal weak pointer. So that
  * extractGeographicBaseObject() can later return a shared pointer on itself.
  */
@@ -282,13 +293,16 @@ const BaseObjectNNPtr *PropertyMap::get(const std::string &key) const {
     }
     return nullptr;
 }
+//! @endcond
+
 // ---------------------------------------------------------------------------
 
 //! @cond Doxygen_Suppress
 void PropertyMap::unset(const std::string &key) {
-    for (auto iter = d->list_.begin(); iter != d->list_.end(); ++iter) {
+    auto &list = d->list_;
+    for (auto iter = list.begin(); iter != list.end(); ++iter) {
         if (iter->first == key) {
-            d->list_.erase(iter);
+            list.erase(iter);
             return;
         }
     }
