@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.5.1.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,8 +31,11 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_PJ_WKT2_SRC_WKT2_GENERATED_PARSER_H_INCLUDED
-# define YY_PJ_WKT2_SRC_WKT2_GENERATED_PARSER_H_INCLUDED
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
+#ifndef YY_PJ_WKT2_WKT2_GENERATED_PARSER_H_INCLUDED
+# define YY_PJ_WKT2_WKT2_GENERATED_PARSER_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -191,7 +195,8 @@ extern int pj_wkt2_debug;
     T_PAST = 400,
     T_UNSPECIFIED = 401,
     T_STRING = 402,
-    T_UNSIGNED_INTEGER_DIFFERENT_ONE_TWO_THREE = 403
+    T_UNSIGNED_INTEGER_DIFFERENT_ONE_TWO_THREE = 403,
+    T_EPSG_CODE_FOR_INTERPOLATION = 404
   };
 #endif
 
@@ -206,4 +211,4 @@ typedef int YYSTYPE;
 
 int pj_wkt2_parse (pj_wkt2_parse_context *context);
 
-#endif /* !YY_PJ_WKT2_SRC_WKT2_GENERATED_PARSER_H_INCLUDED  */
+#endif /* !YY_PJ_WKT2_WKT2_GENERATED_PARSER_H_INCLUDED  */
